@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #ifndef R2_CORE_H
 #define R2_CORE_H
@@ -30,13 +30,17 @@
 #include "r_util/r_print.h"
 #include "r_crypto.h"
 #include "r_bind.h"
+#include "r_codemeta.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 R_LIB_VERSION_HEADER(r_core);
 
+#define R_CORE_CMD_OK 0
+#define R_CORE_CMD_INVALID -1
 #define R_CORE_CMD_EXIT -2
+
 #define R_CORE_BLOCKSIZE 0x100
 #define R_CORE_BLOCKSIZE_MAX 0x3200000 /* 32MB */
 
@@ -52,6 +56,7 @@ R_LIB_VERSION_HEADER(r_core);
 #define R_FLAGS_FS_FUNCTIONS "functions"
 #define R_FLAGS_FS_IMPORTS "imports"
 #define R_FLAGS_FS_RELOCS "relocs"
+#define R_FLAGS_FS_REGISTERS "registers"
 #define R_FLAGS_FS_RESOURCES "resources"
 #define R_FLAGS_FS_SECTIONS "sections"
 #define R_FLAGS_FS_SEGMENTS "segments"
@@ -99,6 +104,17 @@ typedef enum {
 #define R_CORE_VISUAL_MODE_PXR   9
 */
 
+typedef struct r_core_plugin_t {
+	const char *name;
+	const char *desc;
+	const char *license;
+	const char *author;
+	const char *version;
+	RCmdCb call; // returns true if command was handled, false otherwise.
+	RCmdCb init;
+	RCmdCb fini;
+} RCorePlugin;
+
 typedef struct r_core_rtr_host_t {
 	int proto;
 	char host[512];
@@ -131,15 +147,6 @@ typedef struct r_core_log_t {
 	RStrpool *sp;
 } RCoreLog;
 
-typedef struct r_core_file_t {
-	int dbg;
-	int fd;
-	RBinBind binb;
-	const struct r_core_t *core;
-	ut8 alive;
-} RCoreFile;
-
-
 typedef struct r_core_times_t {
 	ut64 loadlibs_init_time;
 	ut64 loadlibs_time;
@@ -159,6 +166,7 @@ typedef enum r_core_autocomplete_types_t {
 	R_CORE_AUTOCMPLT_FCN,
 	R_CORE_AUTOCMPLT_ZIGN,
 	R_CORE_AUTOCMPLT_EVAL,
+	R_CORE_AUTOCMPLT_VARS,
 	R_CORE_AUTOCMPLT_PRJT,
 	R_CORE_AUTOCMPLT_MINS,
 	R_CORE_AUTOCMPLT_BRKP,
@@ -219,7 +227,7 @@ typedef struct {
 	char *cmd;
 } RCoreGadget;
 
-R_API void r_core_gadget_free (RCoreGadget *g);
+R_API void r_core_gadget_free(RCoreGadget *g);
 
 typedef struct r_core_tasks_t {
 	int task_id_next;
@@ -234,9 +242,24 @@ typedef struct r_core_tasks_t {
 	bool oneshot_running;
 } RCoreTaskScheduler;
 
-typedef struct r_core_t {
+typedef struct r_core_project_t {
+	char *name;
+	char *path;
+} RProject;
+
+R_API RProject *r_project_new(void);
+R_API bool r_project_rename(RProject *p, const char *newname);
+R_API bool r_project_is_git(RProject *p);
+R_API void r_project_close(RProject *p);
+R_API bool r_project_open(RProject *p, const char *prjname, const char *path);
+R_API void r_project_save(RProject *p);
+R_API void r_project_free(RProject *p);
+R_API bool r_project_is_loaded(RProject *p);
+
+struct r_core_t {
 	RBin *bin;
 	RConfig *config;
+	RProject *prj;
 	ut64 offset; // current seek
 	ut64 prompt_offset; // temporarily set to offset to have $$ in expressions always stay the same during temp seeks
 	ut32 blocksize;
@@ -250,8 +273,6 @@ typedef struct r_core_t {
 	/* files */
 	RCons *cons;
 	RIO *io;
-	RCoreFile *file;
-	RList *files;
 	RNum *num;
 	ut64 rc; // command's return code .. related to num->value;
 	RLib *lib;
@@ -259,7 +280,7 @@ typedef struct r_core_t {
 	RCmdDescriptor root_cmd_descriptor;
 	RList/*<RCmdDescriptor>*/ *cmd_descriptors;
 	RAnal *anal;
-	RAsm *assembler;
+	RAsm *rasm;
 	/* ^^ */
 	RCoreTimes *times;
 	RParse *parser;
@@ -305,7 +326,7 @@ typedef struct r_core_t {
 	int incomment;
 	int curtab; // current tab
 	int seltab; // selected tab
-	int cmdremote;
+	char *cmdremote;
 	char *lastsearch;
 	char *cmdfilter;
 	bool break_loop;
@@ -317,6 +338,7 @@ typedef struct r_core_t {
 	char *table_query;
 	int sync_index; // used for http.sync and T=
 	struct r_core_t *c2;
+	RTable *table;
 	RCoreAutocomplete *autocomplete;
 	int autocomplete_type;
 	int maxtab;
@@ -326,6 +348,9 @@ typedef struct r_core_t {
 	bool log_events; // core.c:cb_event_handler : log actions from events if cfg.log.events is set
 	RList *ropchain;
 	bool use_tree_sitter_r2cmd;
+	char *theme;
+	bool marks_init;
+	ut64 marks[UT8_MAX + 1];
 
 	RMainCallback r_main_radare2;
 	// int (*r_main_radare2)(int argc, char **argv);
@@ -336,7 +361,7 @@ typedef struct r_core_t {
 	int (*r_main_ragg2)(int argc, const char **argv);
 	int (*r_main_rasm2)(int argc, const char **argv);
 	int (*r_main_rax2)(int argc, const char **argv);
-} RCore;
+};
 
 // maybe move into RAnal
 typedef struct r_core_item_t {
@@ -352,9 +377,8 @@ typedef struct r_core_item_t {
 	char *fcnname;
 } RCoreItem;
 
-
-R_API RCoreItem *r_core_item_at (RCore *core, ut64 addr);
-R_API void r_core_item_free (RCoreItem *ci);
+R_API RCoreItem *r_core_item_at(RCore *core, ut64 addr);
+R_API void r_core_item_free(RCoreItem *ci);
 
 R_API int r_core_bind(RCore *core, RCoreBind *bnd);
 
@@ -371,7 +395,7 @@ typedef int (*RCoreSearchCallback)(RCore *core, ut64 from, ut8 *buf, int len);
 #ifdef R_API
 //#define r_core_ncast(x) (RCore*)(size_t)(x)
 R_API RList *r_core_list_themes(RCore *core);
-R_API char *r_core_get_theme(void);
+R_API char *r_core_get_theme(RCore *core);
 R_API const char *r_core_get_section_name(RCore *core, ut64 addr);
 R_API RCons *r_core_get_cons(RCore *core);
 R_API RBin *r_core_get_bin(RCore *core);
@@ -394,18 +418,18 @@ R_API int r_core_lines_initcache (RCore *core, ut64 start_addr, ut64 end_addr);
 R_API int r_core_lines_currline (RCore *core);
 R_API void r_core_prompt_loop(RCore *core);
 R_API ut64 r_core_pava(RCore *core, ut64 addr);
-R_API int r_core_cmd(RCore *core, const char *cmd, int log);
+R_API int r_core_cmd(RCore *core, const char *cmd, bool log);
 R_API int r_core_cmd_task_sync(RCore *core, const char *cmd, bool log);
-R_API char *r_core_editor (const RCore *core, const char *file, const char *str);
+R_API char *r_core_editor(const RCore *core, const char *file, const char *str);
 R_API int r_core_fgets(char *buf, int len);
 R_API RFlagItem *r_core_flag_get_by_spaces(RFlag *f, ut64 off);
-R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
+R_API int r_core_cmdf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_core_flush(RCore *core, const char *cmd);
 R_API int r_core_cmd0(RCore *core, const char *cmd);
 R_API void r_core_cmd_init(RCore *core);
 R_API int r_core_cmd_pipe(RCore *core, char *radare_cmd, char *shell_cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
-R_API char *r_core_cmd_strf(RCore *core, const char *fmt, ...);
+R_API char *r_core_cmd_strf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_core_cmd_str_pipe(RCore *core, const char *cmd);
 R_API int r_core_cmd_file(RCore *core, const char *file);
 R_API int r_core_cmd_lines(RCore *core, const char *lines);
@@ -441,7 +465,6 @@ R_API int r_core_visual_anal_classes(RCore *core);
 R_API int r_core_visual_types(RCore *core);
 R_API int r_core_visual(RCore *core, const char *input);
 R_API int r_core_visual_graph(RCore *core, RAGraph *g, RAnalFunction *_fcn, int is_interactive);
-R_API bool r_core_visual_panels_root(RCore *core, RPanelsRoot *panels_root);
 R_API void r_core_visual_browse(RCore *core, const char *arg);
 R_API int r_core_visual_cmd(RCore *core, const char *arg);
 R_API void r_core_visual_seek_animation (RCore *core, ut64 addr);
@@ -460,13 +483,18 @@ R_API ut64 r_core_get_asmqjmps(RCore *core, const char *str);
 R_API void r_core_set_asmqjmps(RCore *core, char *str, size_t len, int i);
 R_API char* r_core_add_asmqjmp(RCore *core, ut64 addr);
 
+// core/panels.c
+R_API bool r_core_panels_root(RCore *core, RPanelsRoot *panels_root);
+R_API void r_core_panels_save(RCore *core, const char *_name);
+R_API bool r_core_panels_load(RCore *core, const char *_name);
+
 R_API void r_core_anal_type_init(RCore *core);
+R_API char *r_core_anal_hasrefs_to_depth(RCore *core, ut64 value, PJ *pj, int depth);
 R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn);
 R_API void r_core_anal_inflags (RCore *core, const char *glob);
-R_API int cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
+R_API bool cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
 R_API void r_core_anal_cc_init(RCore *core);
 R_API void r_core_anal_paths(RCore *core, ut64 from, ut64 to, bool followCalls, int followDepth, bool is_json);
-R_API void r_core_anal_esil_graph(RCore *core, const char *expr);
 
 R_API void r_core_list_io(RCore *core);
 R_API RListInfo *r_listinfo_new (const char *name, RInterval pitv, RInterval vitv, int perm, const char *extra);
@@ -481,27 +509,17 @@ R_API void r_core_visual_mark_reset(RCore *core);
 
 R_API int r_core_search_cb(RCore *core, ut64 from, ut64 to, RCoreSearchCallback cb);
 R_API bool r_core_serve(RCore *core, RIODesc *fd);
-R_API int r_core_file_reopen(RCore *core, const char *args, int perm, int binload);
+
+// RCoreFile APIs (bind, riodesc + rbinfile)
+R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int binload);
 R_API void r_core_file_reopen_debug(RCore *core, const char *args);
 R_API void r_core_file_reopen_remote_debug(RCore *core, char *uri, ut64 addr);
-R_API RCoreFile * r_core_file_find_by_fd(RCore* core, ut64 fd);
-R_API RCoreFile * r_core_file_find_by_name (RCore * core, const char * name);
-R_API RCoreFile * r_core_file_cur (RCore *r);
-R_API int r_core_file_set_by_fd(RCore *core, ut64 fd);
-R_API int r_core_file_set_by_name(RCore *core, const char * name);
-R_API int r_core_file_set_by_file (RCore * core, RCoreFile *cf);
-R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach);
-
-R_API void r_core_file_free(RCoreFile *cf);
-R_API RCoreFile *r_core_file_open(RCore *core, const char *file, int flags, ut64 loadaddr);
-R_API RCoreFile *r_core_file_open_many(RCore *r, const char *file, int flags, ut64 loadaddr);
-R_API RCoreFile *r_core_file_get_by_fd(RCore *core, int fd);
-R_API int r_core_file_close(RCore *core, RCoreFile *fh);
-R_API bool r_core_file_close_fd(RCore *core, int fd);
+R_API RIODesc *r_core_file_open(RCore *core, const char *file, int flags, ut64 loadaddr);
+R_API RIODesc *r_core_file_open_many(RCore *r, const char *file, int flags, ut64 loadaddr);
 R_API bool r_core_file_close_all_but(RCore *core);
-R_API int r_core_file_list(RCore *core, int mode);
-R_API int r_core_file_binlist(RCore *core);
-R_API bool r_core_file_bin_raise(RCore *core, ut32 num);
+
+
+R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach);
 R_API int r_core_seek_delta(RCore *core, st64 addr);
 R_API bool r_core_extend_at(RCore *core, ut64 addr, int size);
 R_API bool r_core_write_at(RCore *core, ut64 addr, const ut8 *buf, int size);
@@ -509,7 +527,6 @@ R_API int r_core_write_op(RCore *core, const char *arg, char op);
 R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op);
 R_API int r_core_set_file_by_fd (RCore * core, ut64 bin_fd);
 R_API int r_core_set_file_by_name (RBin * bin, const char * name);
-R_API ut32 r_core_file_cur_fd (RCore *core);
 
 R_API void r_core_debug_rr (RCore *core, RReg *reg, int mode);
 
@@ -519,8 +536,10 @@ R_API void r_core_fortune_list(RCore *core);
 R_API void r_core_fortune_print_random(RCore *core);
 
 /* project */
+#if 0
 R_API bool r_core_project_load(RCore *core, const char *prjfile, const char *rcfile);
 R_API RThread *r_core_project_load_bg(RCore *core, const char *prjfile, const char *rcfile);
+#endif
 R_API void r_core_project_execute_cmds(RCore *core, const char *prjfile);
 
 #define R_CORE_FOREIGN_ADDR -1
@@ -547,9 +566,9 @@ R_API int r_core_yank_file_all (RCore *core, const char *input);
 #define R_CORE_LOADLIBS_ALL UT32_MAX
 
 R_API void r_core_loadlibs_init(RCore *core);
-R_API int r_core_loadlibs(RCore *core, int where, const char *path);
+R_API bool r_core_loadlibs(RCore *core, int where, const char *path);
 R_API int r_core_cmd_buffer(RCore *core, const char *buf);
-R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
+R_API int r_core_cmdf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_core_cmd0(RCore *core, const char *cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
 R_API int r_core_cmd_foreach(RCore *core, const char *cmd, char *each);
@@ -571,16 +590,16 @@ R_API void r_core_anal_esil(RCore *core, const char *str, const char *addr);
 R_API void r_core_anal_fcn_merge (RCore *core, ut64 addr, ut64 addr2);
 R_API const char *r_core_anal_optype_colorfor(RCore *core, ut64 addr, bool verbose);
 R_API ut64 r_core_anal_address (RCore *core, ut64 addr);
-R_API void r_core_anal_undefine (RCore *core, ut64 off);
-R_API void r_core_anal_hint_print (RAnal* a, ut64 addr, int mode);
-R_API void r_core_anal_hint_list (RAnal *a, int mode);
+R_API void r_core_anal_undefine(RCore *core, ut64 off);
+R_API void r_core_anal_hint_print(RAnal* a, ut64 addr, int mode);
+R_API void r_core_anal_hint_list(RAnal *a, int mode);
 R_API int r_core_anal_search(RCore *core, ut64 from, ut64 to, ut64 ref, int mode);
-R_API int r_core_anal_search_xrefs(RCore *core, ut64 from, ut64 to, int rad);
-R_API int r_core_anal_data (RCore *core, ut64 addr, int count, int depth, int wordsize);
+R_API int r_core_anal_search_xrefs(RCore *core, ut64 from, ut64 to, PJ *pj, int rad);
+R_API int r_core_anal_data(RCore *core, ut64 addr, int count, int depth, int wordsize);
 R_API void r_core_anal_datarefs(RCore *core, ut64 addr);
 R_API void r_core_anal_coderefs(RCore *core, ut64 addr);
-R_API void r_core_anal_codexrefs(RCore *core, ut64 addr);
-R_API void r_core_anal_importxrefs(RCore *core);
+R_API RGraph/*<RGraphNodeInfo>*/ *r_core_anal_codexrefs(RCore *core, ut64 addr);
+R_API RGraph/*<RGraphNodeInfo>*/ *r_core_anal_importxrefs(RCore *core);
 R_API void r_core_anal_callgraph(RCore *core, ut64 addr, int fmt);
 R_API int r_core_anal_refs(RCore *core, const char *input);
 R_API void r_core_agraph_print(RCore *core, int use_utf, const char *input);
@@ -610,9 +629,9 @@ R_API RList *r_core_anal_fcn_get_calls (RCore *core, RAnalFunction *fcn); // get
 
 /*tp.c*/
 R_API void r_core_anal_type_match(RCore *core, RAnalFunction *fcn);
-R_API RStrBuf *var_get_constraint (RAnal *a, RAnalVar *var);
 
 /* asm.c */
+#define R_MIDFLAGS_HIDE 0
 #define R_MIDFLAGS_SHOW 1
 #define R_MIDFLAGS_REALIGN 2
 #define R_MIDFLAGS_SYMALIGN 3
@@ -625,7 +644,7 @@ typedef struct r_core_asm_hit {
 } RCoreAsmHit;
 
 R_API RBuffer *r_core_syscall (RCore *core, const char *name, const char *args);
-R_API RBuffer *r_core_syscallf (RCore *core, const char *name, const char *fmt, ...);
+R_API RBuffer *r_core_syscallf (RCore *core, const char *name, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
 R_API RCoreAsmHit *r_core_asm_hit_new(void);
 R_API RList *r_core_asm_hit_list_new(void);
 R_API void r_core_asm_hit_free(void *_hit);
@@ -638,15 +657,20 @@ R_API RList *r_core_asm_back_disassemble_byte (RCore *core, ut64 addr, int len,
 R_API ut32 r_core_asm_bwdis_len (RCore* core, int* len, ut64* start_addr, ut32 l);
 R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int len, int lines, int invbreak, int nbytes, bool json, PJ *pj, RAnalFunction *pdf);
 R_API int r_core_print_disasm_json(RCore *core, ut64 addr, ut8 *buf, int len, int lines, PJ *pj);
-R_API int r_core_print_disasm_instructions (RCore *core, int len, int l);
-R_API int r_core_print_disasm_all (RCore *core, ut64 addr, int l, int len, int mode);
+R_API int r_core_print_disasm_instructions_with_buf(RCore *core, ut64 address, ut8 *buf, int nb_bytes, int nb_opcodes);
+R_API int r_core_print_disasm_instructions(RCore *core, int nb_bytes, int nb_opcodes);
+R_API int r_core_print_disasm_all(RCore *core, ut64 addr, int l, int len, int mode);
+R_API int r_core_disasm_pdi_with_buf(RCore *core, ut64 address, ut8 *buf, ut32 nb_opcodes, ut32 nb_bytes, int fmt);
 R_API int r_core_disasm_pdi(RCore *core, int nb_opcodes, int nb_bytes, int fmt);
+R_API int r_core_disasm_pde(RCore *core, int nb_opcodes, int mode);
 R_API int r_core_print_fcn_disasm(RPrint *p, RCore *core, ut64 addr, int l, int invbreak, int cbytes);
 R_API int r_core_get_prc_cols(RCore *core);
 R_API int r_core_flag_in_middle(RCore *core, ut64 at, int oplen, int *midflags);
 R_API int r_core_bb_starts_in_middle(RCore *core, ut64 at, int oplen);
 
+// both do the same, we should get rid of one of them
 R_API bool r_core_bin_raise (RCore *core, ut32 bfid);
+R_API bool r_core_bin_set_cur(RCore *core, RBinFile *binfile);
 
 R_API int r_core_bin_set_env (RCore *r, RBinFile *binfile);
 R_API int r_core_bin_set_by_fd (RCore *core, ut64 bin_fd);
@@ -654,7 +678,7 @@ R_API int r_core_bin_set_by_name (RCore *core, const char *name);
 R_API int r_core_bin_reload(RCore *core, const char *file, ut64 baseaddr);
 R_API bool r_core_bin_load(RCore *core, const char *file, ut64 baseaddr);
 R_API int r_core_bin_rebase(RCore *core, ut64 baddr);
-R_API void r_core_bin_export_info_rad(RCore *core);
+R_API void r_core_bin_export_info(RCore *core, int mode);
 R_API int r_core_bin_list(RCore *core, int mode);
 R_API bool r_core_bin_delete (RCore *core, ut32 binfile_idx);
 R_API ut64 r_core_bin_impaddr(RBin *bin, int va, const char *name);
@@ -668,13 +692,13 @@ R_API int r_core_zdiff(RCore *c, RCore *c2);
 R_API int r_core_gdiff(RCore *core1, RCore *core2);
 R_API int r_core_gdiff_fcn(RCore *c, ut64 addr, ut64 addr2);
 
-R_API bool r_core_project_open(RCore *core, const char *file, bool thready);
+R_API bool r_core_project_open(RCore *core, const char *file);
 R_API int r_core_project_cat(RCore *core, const char *name);
 R_API int r_core_project_delete(RCore *core, const char *prjfile);
 R_API int r_core_project_list(RCore *core, int mode);
-R_API bool r_core_project_save_rdb(RCore *core, const char *file, int opts);
+R_API bool r_core_project_save_script(RCore *core, const char *file, int opts);
 R_API bool r_core_project_save(RCore *core, const char *file);
-R_API char *r_core_project_info(RCore *core, const char *file);
+R_API char *r_core_project_name(RCore *core, const char *file);
 R_API char *r_core_project_notes_file (RCore *core, const char *file);
 
 R_API char *r_core_sysenv_begin(RCore *core, const char *cmd);
@@ -731,11 +755,11 @@ typedef struct r_core_bin_filter_t {
 	const char *name;
 } RCoreBinFilter;
 
-R_API int r_core_bin_info (RCore *core, int action, int mode, int va, RCoreBinFilter *filter, const char *chksum);
+R_API int r_core_bin_info (RCore *core, int action, PJ *pj, int mode, int va, RCoreBinFilter *filter, const char *chksum);
 R_API int r_core_bin_set_arch_bits (RCore *r, const char *name, const char * arch, ut16 bits);
 R_API int r_core_bin_update_arch_bits (RCore *r);
 R_API char *r_core_bin_method_flags_str(ut64 flags, int mode);
-R_API int r_core_pdb_info(RCore *core, const char *file, ut64 baddr, int mode);
+R_API bool r_core_pdb_info(RCore *core, const char *file, PJ *pj, int mode);
 
 /* rtr */
 R_API int r_core_rtr_cmds (RCore *core, const char *port);
@@ -808,6 +832,7 @@ R_API void r_core_log_del(RCore *core, int n);
 // TODO MOVE SOMEWHERE ELSE
 typedef char *(*PrintItemCallback)(void *user, void *p, bool selected);
 R_API char *r_str_widget_list(void *user, RList *list, int rows, int cur, PrintItemCallback cb);
+R_API PJ *r_core_pj_new(RCore *core);
 /* help */
 R_API void r_core_cmd_help(const RCore *core, const char * help[]);
 
@@ -830,7 +855,7 @@ typedef struct {
 
 R_API bool core_anal_bbs(RCore *core, const char* input);
 R_API bool core_anal_bbs_range (RCore *core, const char* input);
-R_API char *r_core_anal_hasrefs(RCore *core, ut64 value, bool verbose);
+R_API char *r_core_anal_hasrefs(RCore *core, ut64 value, int mode);
 R_API char *r_core_anal_get_comments(RCore *core, ut64 addr);
 R_API RCoreAnalStats* r_core_anal_get_stats (RCore *a, ut64 from, ut64 to, ut64 step);
 R_API void r_core_anal_stats_free (RCoreAnalStats *s);
@@ -844,6 +869,9 @@ R_API int r_line_hist_offset_down(RLine *line);
 
 // TODO : move into debug or syscall++
 R_API char *cmd_syscall_dostr(RCore *core, st64 num, ut64 addr);
+R_API void cmd_agfb(RCore *core);
+R_API void cmd_agfb2(RCore *core, const char *s);
+R_API void cmd_agfb3(RCore *core, const char *s, int x, int y);
 
 /* tasks */
 
@@ -878,13 +906,13 @@ typedef struct r_core_task_t {
 typedef void (*RCoreTaskOneShot)(void *);
 
 R_API void r_core_echo(RCore *core, const char *msg);
-R_API RTable *r_core_table(RCore *core);
+R_API RTable *r_core_table(RCore *core, const char *name);
 
 R_API void r_core_task_scheduler_init (RCoreTaskScheduler *tasks, RCore *core);
 R_API void r_core_task_scheduler_fini (RCoreTaskScheduler *tasks);
 R_API RCoreTask *r_core_task_get(RCoreTaskScheduler *scheduler, int id);
 R_API RCoreTask *r_core_task_get_incref(RCoreTaskScheduler *scheduler, int id);
-R_API void r_core_task_print(RCore *core, RCoreTask *task, int mode);
+R_API void r_core_task_print(RCore *core, RCoreTask *task, PJ *pj, int mode);
 R_API void r_core_task_list(RCore *core, int mode);
 R_API int r_core_task_running_tasks_count(RCoreTaskScheduler *scheduler);
 R_API const char *r_core_task_status(RCoreTask *task);
@@ -905,8 +933,7 @@ R_API int r_core_task_del(RCoreTaskScheduler *scheduler, int id);
 R_API void r_core_task_del_all_done(RCoreTaskScheduler *scheduler);
 R_API RCoreTask *r_core_task_self(RCoreTaskScheduler *scheduler);
 R_API void r_core_task_join(RCoreTaskScheduler *scheduler, RCoreTask *current, int id);
-typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize,
-		int count, void *cb_user);
+typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize, void *cb_user);
 R_API int r_core_search_value_in_range (RCore *core, RInterval search_itv,
 		ut64 vmin, ut64 vmax, int vsize, inRangeCb cb, void *cb_user);
 
@@ -920,6 +947,44 @@ R_API void r_core_anal_propagate_noreturn(RCore *core, ut64 addr);
 /* PLUGINS */
 extern RCorePlugin r_core_plugin_java;
 extern RCorePlugin r_core_plugin_a2f;
+extern RCorePlugin r_core_plugin_sixref;
+R_API bool r_core_plugin_init(RCmd *cmd);
+R_API bool r_core_plugin_add(RCmd *cmd, RCorePlugin *plugin);
+R_API bool r_core_plugin_check(RCmd *cmd, const char *a0);
+R_API bool r_core_plugin_fini(RCmd *cmd);
+
+
+/* DECOMPILER PRINTING FUNCTIONS */
+/**
+ * @brief Prints the data contained in the specified RAnnotatedCode in JSON format.
+ * 
+ * The function will print the output in console using the function r_cons_printf();
+ * 
+ * @param code Pointer to a RAnnotatedCode.
+ */
+R_API void r_codemeta_print_json(RCodeMeta *code);
+/**
+ * @brief Prints the decompiled code from the specified RAnnotatedCode.
+ * 
+ * This function is used for printing the output of commands pdg and pdgo.
+ * It can print the decompiled code with or without offsets. If line_offsets is a null pointer,
+ * the output will be printed without offsets (pdg), otherwise, the output will be
+ * printed with offsets.
+ * This function will print the output in console using the function r_cons_printf();
+ * 
+ * @param code Pointer to a RAnnotatedCode.
+ * @param line_offsets Pointer to a @ref RVector that contains offsets for the decompiled code.
+ */
+R_API void r_codemeta_print(RCodeMeta *code, RVector *line_offsets);
+/**
+ * @brief  Prints the decompiled code as comments
+ * 
+ * This function is used for the output of command pdg*
+ * Output will be printed in console using the function r_cons_printf();
+ * 
+ * @param code Pointer to a RAnnotatedCode.
+ */
+R_API void r_codemeta_print_comment_cmds(RCodeMeta *code);
 
 #endif
 
