@@ -17,6 +17,7 @@ extern "C" {
 
 // Singleton can be zero initialized
 typedef struct r_rb_node_t {
+	struct r_rb_node_t *parent;
 	struct r_rb_node_t *child[2];
 	bool red;
 } RBNode;
@@ -109,8 +110,12 @@ R_API RContRBTree *r_rbtree_cont_new(void);
 R_API RContRBTree *r_rbtree_cont_newf(RContRBFree f);
 R_API bool r_rbtree_cont_insert(RContRBTree *tree, void *data, RContRBCmp cmp, void *user);
 R_API bool r_rbtree_cont_delete(RContRBTree *tree, void *data, RContRBCmp cmp, void *user);
+R_API RContRBNode *r_rbtree_cont_find_node(RContRBTree *tree, void *data, RContRBCmp cmp, void *user);
+R_API RContRBNode *r_rbtree_cont_node_next(RContRBNode *node);
+R_API RContRBNode *r_rbtree_cont_node_prev(RContRBNode *node);
 R_API void *r_rbtree_cont_find(RContRBTree *tree, void *data, RContRBCmp cmp, void *user);
 R_API void *r_rbtree_cont_first(RContRBTree *tree);
+R_API void *r_rbtree_cont_last(RContRBTree *tree);
 
 #define r_rbtree_cont_foreach(tree, it, dat) \
 	for ((it) = r_rbtree_first ((tree)->root ? &(tree)->root->node : NULL); r_rbtree_iter_has(&it) && (dat = r_rbtree_iter_get (&it, RContRBNode, node)->data); r_rbtree_iter_next (&(it)))
